
# q-expansion of newform 288.4.d.c, downloaded from the LMFDB on 18 January 2025.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 288
weight = 4
poly_data = [64, 0, -6, 0, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0, 0, 0], 1], [[0, 2, 0, 1], 4], [[0, 28, 0, -2], 1], [[-24, 0, 8, 0], 1]]

hecke_ring_character_values = [[127, [1, 0, 0, 0]], [37, [-1, 0, 0, 0]], [65, [1, 0, 0, 0]]]
aps_data = [[0, 0, 0, 0], [0, 0, 0, 0], [0, 1, 0, 0], [-10, 0, 0, 0], [0, -6, 0, 0], [0, 0, 0, 1], [0, 0, -1, 0], [0, 0, 0, -2], [0, 0, -2, 0], [0, -39, 0, 0], [-62, 0, 0, 0], [0, 0, 0, 1], [0, 0, -5, 0], [0, 0, 0, 2], [0, 0, -6, 0], [0, -21, 0, 0], [0, -116, 0, 0], [0, 0, 0, -9], [0, 0, 0, 12], [0, 0, 0, 0], [30, 0, 0, 0], [-94, 0, 0, 0], [0, 106, 0, 0], [0, 0, 10, 0], [130, 0, 0, 0], [0, -89, 0, 0], [570, 0, 0, 0], [0, 204, 0, 0], [0, 0, 0, 29], [0, 0, 18, 0], [-2530, 0, 0, 0], [0, 40, 0, 0], [0, 0, 9, 0], [0, 0, 0, 4], [0, 465, 0, 0], [2262, 0, 0, 0], [0, 0, 0, -29], [0, 0, 0, -34], [0, 0, 14, 0], [0, -225, 0, 0], [0, -20, 0, 0], [0, 0, 0, -1], [0, 0, -50, 0], [-1350, 0, 0, 0], [0, -229, 0, 0], [3194, 0, 0, 0], [0, 0, 0, 92], [-5330, 0, 0, 0], [0, -754, 0, 0], [0, 0, 0, -61], [0, 0, -2, 0], [0, 0, -30, 0], [-1162, 0, 0, 0], [0, -102, 0, 0], [0, 0, -24, 0], [0, 0, -88, 0], [0, 957, 0, 0], [-6402, 0, 0, 0], [0, 0, 0, 35], [0, 0, -120, 0], [0, 0, 0, -88], [0, -837, 0, 0], [0, 0, 0, -84], [0, 0, -20, 0], [6350, 0, 0, 0], [0, -275, 0, 0], [0, 0, 0, 56], [-1430, 0, 0, 0], [0, 1598, 0, 0], [0, 0, 0, 19], [0, 0, -18, 0], [0, 0, 160, 0], [-5070, 0, 0, 0], [0, 0, 0, 41], [0, 0, 0, 130], [0, 0, 78, 0], [0, 341, 0, 0], [0, 0, 0, -205], [0, 0, 45, 0], [3526, 0, 0, 0], [0, 390, 0, 0], [0, 0, 0, -229], [0, 0, 90, 0], [7790, 0, 0, 0], [9354, 0, 0, 0], [0, -1026, 0, 0], [0, 0, 135, 0], [-12010, 0, 0, 0], [0, -1473, 0, 0], [-14770, 0, 0, 0], [0, 1382, 0, 0], [0, 0, -40, 0], [9910, 0, 0, 0], [0, -536, 0, 0], [0, 0, 0, -72], [0, 0, 12, 0], [0, -135, 0, 0], [0, 0, -105, 0], [0, 0, 0, -174], [0, 0, 0, -263], [0, 0, 0, -38], [0, -45, 0, 0], [0, -2898, 0, 0], [0, 0, -105, 0], [0, 0, 0, -140], [-6330, 0, 0, 0], [0, 304, 0, 0], [0, 0, -138, 0], [0, 0, 10, 0], [18578, 0, 0, 0], [8030, 0, 0, 0], [0, 0, 0, 65], [0, 0, -4, 0], [0, 0, 0, 24], [12902, 0, 0, 0], [0, 0, -265, 0], [0, 0, 0, -502], [0, 0, 186, 0], [0, 2919, 0, 0], [0, 816, 0, 0], [0, 0, 0, -227], [-15010, 0, 0, 0], [0, 993, 0, 0], [0, 1978, 0, 0], [0, 0, 0, 478], [0, -3131, 0, 0], [0, 0, 0, -217], [0, 0, -20, 0], [-23850, 0, 0, 0], [0, 0, 0, 37], [0, 0, 0, 44], [0, 0, 38, 0], [20578, 0, 0, 0], [0, 0, 0, -345], [0, 0, -295, 0], [-6854, 0, 0, 0], [0, 6151, 0, 0], [0, 0, 0, -222], [0, 2171, 0, 0], [0, 0, 405, 0], [0, 0, 0, -466], [0, 4159, 0, 0], [-22970, 0, 0, 0], [0, -1212, 0, 0], [0, 0, 0, 293], [0, 0, 440, 0], [0, 0, 0, 777], [0, 0, 591, 0], [0, 0, 0, -130], [0, 0, 282, 0], [0, 0, 0, -289], [0, 0, -560, 0], [0, 0, 0, -82], [0, 0, 156, 0], [0, 0, 0, -218], [0, 0, -270, 0], [-22746, 0, 0, 0], [0, 0, 185, 0], [-27850, 0, 0, 0], [0, -5817, 0, 0], [0, 6756, 0, 0], [0, 0, -333, 0], [-10550, 0, 0, 0], [0, -8382, 0, 0], [0, 0, 461, 0], [0, 0, 452, 0], [8818, 0, 0, 0], [0, 0, 0, -423]]
